"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const getLastBuffers_1 = __importDefault(require("../functions/getLastBuffers"));
const sortBuffers_1 = __importDefault(require("../functions/sortBuffers"));
const lodash_1 = require("lodash");
const socketDataEncoder_1 = __importDefault(require("../functions/socketDataEncoder"));
const updateSocketData_1 = __importDefault(require("../functions/updateSocketData"));
// import isArrayOfImarketCodes from '../functions/isArrayOfImarketCodes';
/**
 * useWsTicker is a custom hook that connects to a WebSocket API
 * and retrieves real-time ticker data for a given market code.
 * @param targetMarketCodes - Array of market codes to retrieve ticker data for.
 * @param options - `throttle_time` the data update frequency(ms).
 * @throws targetMarketCodes should be React State Value, if not, unexpected errors can occur.
 * @returns Object with the WebSocket object, connection status, and real-time ticker data.
 */
function useWsTicker(targetMarketCodes, onError, options = {}) {
    const { throttle_time = 400, debug = false } = options;
    const SOCKET_URL = 'wss://api.upbit.com/websocket/v1';
    const socket = (0, react_1.useRef)(null);
    const buffer = (0, react_1.useRef)([]);
    const [isConnected, setIsConnected] = (0, react_1.useState)(false);
    const [loadingBuffer, setLoadingBuffer] = (0, react_1.useState)([]);
    const [socketData, setSocketData] = (0, react_1.useState)(null);
    const throttled = (0, lodash_1.throttle)(() => {
        try {
            const lastBuffers = (0, getLastBuffers_1.default)(buffer.current, targetMarketCodes.length);
            const sortedBuffers = (0, sortBuffers_1.default)(lastBuffers, targetMarketCodes);
            sortedBuffers && setLoadingBuffer(sortedBuffers);
            buffer.current = [];
        }
        catch (error) {
            console.error(error);
            return;
        }
    }, throttle_time);
    // socket 세팅
    (0, react_1.useEffect)(() => {
        try {
            // if (
            //   targetMarketCodes.length > 0 &&
            //   !isArrayOfImarketCodes(targetMarketCodes)
            // ) {
            //   throw new Error(
            //     'targetMarketCodes does not have the correct interface',
            //   );
            // }
            if (targetMarketCodes.length > 0 && !socket.current) {
                socket.current = new WebSocket(SOCKET_URL);
                socket.current.binaryType = 'arraybuffer';
                const socketOpenHandler = () => {
                    var _a;
                    setIsConnected(true);
                    if (debug)
                        console.log('[completed connect] | socket Open Type: ', 'ticker');
                    if (((_a = socket.current) === null || _a === void 0 ? void 0 : _a.readyState) == 1) {
                        const sendContent = [
                            { ticket: 'test' },
                            {
                                type: 'ticker',
                                codes: targetMarketCodes.map(code => code.market),
                            },
                        ];
                        socket.current.send(JSON.stringify(sendContent));
                        if (debug)
                            console.log('message sending done');
                    }
                };
                const socketCloseHandler = () => {
                    setIsConnected(false);
                    setLoadingBuffer([]);
                    setSocketData(null);
                    buffer.current = [];
                    if (debug)
                        console.log('connection closed');
                };
                const socketErrorHandler = (event) => {
                    const error = event.error;
                    console.error('[Error]', error);
                };
                const socketMessageHandler = (evt) => {
                    const data = (0, socketDataEncoder_1.default)(evt.data);
                    console.log('data:', data);
                    data && buffer.current.push(data);
                    throttled();
                };
                socket.current.onopen = socketOpenHandler;
                socket.current.onclose = socketCloseHandler;
                socket.current.onerror = socketErrorHandler;
                socket.current.onmessage = socketMessageHandler;
            }
            return () => {
                if (socket.current) {
                    if (socket.current.readyState != 0) {
                        socket.current.close();
                        socket.current = null;
                    }
                }
            };
        }
        catch (error) {
            if (error instanceof Error) {
                if (onError) {
                    onError(error);
                }
                else {
                    console.error(error);
                    throw error;
                }
            }
        }
    }, [targetMarketCodes]);
    (0, react_1.useEffect)(() => {
        try {
            if (loadingBuffer.length > 0) {
                if (!socketData) {
                    setSocketData(loadingBuffer);
                }
                else {
                    setSocketData(prev => {
                        return prev && (0, updateSocketData_1.default)(prev, loadingBuffer);
                    });
                    setLoadingBuffer([]);
                }
            }
        }
        catch (error) {
            console.error(error);
        }
    }, [loadingBuffer]);
    return { socket: socket.current, isConnected, socketData };
}
exports.default = useWsTicker;
